#############
## Script to analyse hashed scData. 
## Produces barcode list / tag that can be used in the follow up analysis. 
#############

# Input: Script takes the results of a Cellranger Run and a Citeseq Run as an argument. 
# Analysis: The cells (barcodes) from both runs are intersected and analysed using Seurat.
# Output: One file containing barcodes per tag used in the citeseq run + Overview / QC Plots

# Linda Grob 


library(optparse)
library(Seurat)
library(ggplot2)
library(parallelDist)

option_list = list(
make_option("--citeseq_in", type = "character", help = "Path to the input files that contains the citeseq results per lane."),
make_option("--adt_barcodes_in", type = "character", help = "Path to the input files that contains the adt barcodes."),
make_option("--output_prefix", type = "character", help = "Path that is used as prefix for files.")
)
opt_parser = OptionParser(option_list = option_list)
opt = parse_args(opt_parser)

print("Creating ouput dir:")
print(dirname(opt$output_prefix))
dir.create(dirname(opt$output_prefix))

# Read in data
lanes=strsplit(opt$citeseq_in,",")
CiteSeq <- Read10X(data.dir = lanes[[1]][1], gene.column=1)
CS.HTO <- CreateAssayObject(counts = CiteSeq)
print(str(CS.HTO))

# The newest version of CiteSeq is able to run multiple lanes at the same time. --> To do adapt snake make pipeline and this script.
if (length(lanes)>1){
	for (lane in 2:length(lanes)){
		print(lanes[[1]][lane])
		CiteSeq <- Read10X(data.dir = lanes[lane], gene.column=1)
		CS.HTO.L<-CreateAssayObject(counts = CiteSeq)
		CS.HTO<-merge(CS.HTO,CS.HTO.L)
		head(CS.HTO)
	}
}
CS.HTO <- as.matrix(CS.HTO[-nrow(CS.HTO),])
CellRanger_results.ADT <- Read10X(data.dir = opt$adt_barcodes_in)
# Only continue with barcodes found in citeseq and cellranger analysis
joint.bcs <- intersect(colnames(CellRanger_results.ADT), colnames(CS.HTO))
print(dim(CS.HTO))
CS.HTO <- as.matrix(CS.HTO[, joint.bcs])
print(str(CS.HTO))
is.vector(CS.HTO)
rownames(CS.HTO)<-gsub("[TCGA-]", "",rownames(CS.HTO))
CellRanger_results.ADT <- CellRanger_results.ADT[, joint.bcs]
# Store data in Seurat object
my_sample <- CreateSeuratObject(counts = CellRanger_results.ADT)
print(my_sample)
my_sample <- NormalizeData(my_sample)
my_sample <- FindVariableFeatures(my_sample)
my_sample <- ScaleData(my_sample)
print(str(CS.HTO))
my_sample[["HTO"]] <- CreateAssayObject(counts=CS.HTO)
my_sample <- NormalizeData(my_sample, assay = "HTO", normalization.method = "CLR")
my_sample <- ScaleData(my_sample, assay = "HTO")
## Demux Sample 
## The HTODemux function seems to produce an error if either to many cells are found in the sample or the data is to noisy. 
## Alternative function is used in that case - however then only one list containing all tags is created together with an error message file. 
## If it becomes clear why this happens, potentially adapt the code here. 


function_error<-FALSE
tryCatch({
        my_sample <- HTODemux(my_sample, assay = "HTO", positive.quantile = 0.99)
        },

        warning = function(w) {writeLines("The HTODemux function produced a warning.", paste(opt$output_prefix,"warning.txt"),sep=".")},
        error = function(e) {
                function_error<<-TRUE
                writeLines("The HTODemux function produced an error. Used MULTIseqDemux instead.",paste(trimws(opt$output_prefix),"FUNCTION_ERROR.txt",sep="."))
                my_sample <<- MULTIseqDemux(my_sample, assay="HTO", autoThresh=TRUE)}
)
# Generate Count Table based on the maxID Tag that Seurat assigns
## "Raw Count" shows number of cells / "best-hit-hashtags" without taking singlets / doublets into account.
if (function_error==FALSE){
	table(my_sample$HTO_maxID)
	write.table(table(my_sample$HTO_maxID), file=paste(opt$output_prefix,"raw_tag_counts.txt",sep="."), sep= "\t", col.names = TRUE, row.names = FALSE, quote=FALSE)

	## Cell Counts / hashtag with Doublets & Negative cells treated special.
	table(my_sample$hash.ID)
	write.table(table(my_sample$hash.ID), file=paste(opt$output_prefix,"final_tag_counts.txt",sep="."), sep= "\t", col.names = TRUE, row.names = FALSE, quote=FALSE)

	# RIDGEPLOT
	# Group Cells based on the highest hashtag signal.
	print("Plotting RidgePlot")
	RidgePlot(my_sample, assay = "HTO", features = rownames(my_sample[["HTO"]]), ncol = 2)
	rigdePlot = RidgePlot(my_sample, assay = "HTO", features = rownames(my_sample[["HTO"]]), ncol = 2)
	ggplot2::ggsave(filename= paste(opt$output_prefix,"ridgeplot.png",sep="."),plot = rigdePlot)

	# HTOHeatmap
	#Idents(my_sample) <- "HTO_maxID"
	print("Plotting Heatmap")
	hto_heatmap = HTOHeatmap(my_sample,assay="HTO",raster=FALSE)
	ggplot2::ggsave(filename= paste(opt$output_prefix,"htoheatmap.png",sep="."),plot=hto_heatmap)

	# TSNEs
	# Calculate tSNE embeddings with a distance matrix
	print("Plotting hashtag classification.")
	Idents(my_sample) <- "HTO_classification.global"
	print("t matrix")
	print(t(GetAssayData(object = my_sample, assay = "HTO")))
	print("Calculating dist matrix")
	parD <- parDist(t(GetAssayData(object = my_sample, assay = "HTO")))
	print("parDist worked")
	hto.dist.mtx <- as.matrix(parD)
	print("hto.dist is calculated")
	my_sample <- RunTSNE(my_sample, distance.matrix = hto.dist.mtx, perplexity = 50)
	plot_global<-DimPlot(my_sample,pt.size=2)


	# First, we will remove negative cells from the object. Might not be a good thing to do, thats wy the .subset gets assigned the whole sample currently. Might want to do this later on.
	Idents(my_sample) <- "hash.ID"
	head(my_sample$hash.ID)
	print(levels(my_sample$hash.ID))
	# Write out list with negative cells instead. 
	write.table(names(my_sample$HTO_maxID[which(my_sample$hash.ID =="Negative")]), paste(opt$output_prefix,"barcodes_negatives.txt", sep="."), quote=FALSE, row.names=FALSE, col.names=FALSE)
	#my_sample.subset <- subset(my_sample, idents = "Negative", invert = TRUE) # Depreceated: Negatives can be interesting.  
	
	# Calculate a distance matrix using HTO
	#hto.dist.mtx <- as.matrix(dist(t(GetAssayData(object = my_sample, assay = "HTO"))))
	
	Idents(my_sample) <- "hash.ID"
	my_sample <- RunTSNE(my_sample, distance.matrix = hto.dist.mtx, perplexity = 50)
	plot_hashid<-DimPlot(my_sample,pt.size=2)
	
	Idents(my_sample) <- "HTO_classification"
	my_sample <- RunTSNE(my_sample, distance.matrix = hto.dist.mtx, perplexity = 50)
	plot_classification<-DimPlot(my_sample,pt.size=2,label.size=6)
	
	combined <- CombinePlots(plots=list(plot_global, plot_hashid, plot_classification), ncol=3)
	ggplot2::ggsave(filename= paste(opt$output_prefix,"hashtag_classification.png",sep="."),plot = combined, width=40, height=10)
	
	# Write one barcode list for every tag.
	my_sample.subset <- subset(my_sample, idents = "Negative", invert = TRUE)
	
	for (tag in levels(my_sample$HTO_maxID)){
		print(tag)
		singlets = my_sample.subset[which(my_sample.subset$HTO_classification.global!="Doublet")]
		write.table(names(singlets$HTO_maxID[which(singlets$HTO_maxID ==tag)]), paste(opt$output_prefix,tag,"barcodes_singlets.txt", sep="."), quote=FALSE, row.names=FALSE, col.names=FALSE)
	}
	## Write table with all singlets barcodes.
	levels(my_sample.subset$HTO_classification.global)
	write.table(colnames(my_sample.subset[which(my_sample.subset$HTO_classification.global=="Singlet")]),paste(opt$output_prefix,"all.barcodes_singlets.txt", sep="."),row.names=FALSE, col.names=FALSE,quote=FALSE)
	} else {
	
	# In case HTO demux fails only one list containing all tags is produced at the moment. Either remove that part or add addditional ouput lists.
	my_sample.subset <- subset(my_sample, idents = "Negative", invert = TRUE)
	write.table(colnames(my_sample.subset[which(my_sample.subset$MULTI_ID!="Doublet")]),paste(opt$output_prefix,"barcodes_singlets.txt", sep="."),row.names=FALSE, col.names=FALSE,quote=FALSE)
	}

