##Script to plot heatmaps for the top highly variable genes or a list of manually curated genes

#Example call
#Rscript heatmaps.R --cohort_list RDS_files.txt --colour_config colour_config.merged.txt --selectedGenes genes.txt --add_meta metadata_table.txt --subset_celltype T.cells --outdir ./ --outName T_cells.

 library(Seurat)
  library(plyr)
  library(dplyr)
  library(SingleCellExperiment)
  library(optparse)
  library(reticulate)
  library(ggplot2)
  library(reshape2)
  library(cowplot)
  library(Matrix)
  library(pryr)

option_list <- list(
  make_option("--cohort_list", type = "character", help = "List with paths to the input RDS files that contain SCE objects with GEX results. INPORTANT: GEX and GSVA input files have to be in the same order."),
  make_option("--colour_config", type = "character", help = "Path to config file table that sorts colours to cell types"),
  make_option("--selectedGenes", type = "character", help = "Tab delimited text file with the genes of interest in the first column. No header. The genes will be plotted and given out in chunks, alphabetical order."),
  make_option("--add_meta", type = "character", help = "Tab delimited text file with additional meta data. The first column must have header 'sampleID'. For a default run the full file name of the respective SCE analysis object is required, refer to script documentation on different sampleID formatting. Column headers describe categories. One line per input sample is required. Meta data file itself is optional: without table simply do not use --add_meta"),
  make_option("--subset_celltype", type = "character", help = "Use a subset of the cells, this (major) cell type only"),
  make_option("--outdir", type = "character", help = "Full path to output directory."),
  make_option("--outName", type = "character", help = "Prefix name of output files.")
)
opt_parser <- OptionParser(option_list = option_list)
opt <- parse_args(opt_parser)

# convenience function for string concatenation
"%&%" <- function(a, b) paste(a, b, sep = "")
empty_string <- ""


###################################
###   Set plotting Parameters   ###
###################################

# If one of the values plot_aspect_ratio, chunksize, or number of columns is changed in the following section
# the other values should be checked as well.
# Also, check physical size of the generated output file in the respective plotting section of the script.

# IMPORTANT: when generating UMAP plots with the function RunUMAP() from Seurat the parameters "spread", "min.dist" and "n.neighbors"
# have a great impact on the resulting plot. They should be checked and optimised for each project.

# for all UMAP plots the same aspect ratio is used to have a comparable visualisation
plot_aspect_ratio <- 0.7

### The selected genes and gene sets are plotted in chunks so that the file size is not too large:

# chunksize_genes determines the number of genes shown in one expression plot
chunksize_genes <- 9
# columns_genes determines the number of columns the gene expression plots are arranged in.
## E.g. "columns_genes <- 1" arranges all plots in the output file below one another in a single column.
columns_genes <- 3

# chunk size of gene sets in GSVA plots
chunksize_genesets <- 12
# number of columns in GSVA plots
columns_genesets <- 4

# override the theme change of cowplot
theme_set(theme_grey())
print(Sys.time())


# parameter when using only subset of cell types
# each sample must contain a minimum number of cells of this cell type to be included
# in the cell type specific analysis
min_cells_per_celltype <- 30

# The number of neighbours considered when filtering the anchors for the integration cannot be
# higher than the number of total cells in this sample
# The default is 200
number_neighbours_filtering <- 20


########################
###   Read in data   ###
########################

# read in data from all samples of the cohort
cohort_files <- read.csv(opt$cohort_list, header = FALSE, stringsAsFactors = FALSE)
all_sce_files <- cohort_files$V1
# make a list of all SCE objects
all_sce_objects <- lapply(seq_along(all_sce_files), function(x) {
  print(paste0("Read in sample ", x))
  print(all_sce_files[x])
  my_sce <- readRDS(all_sce_files[x])

  my_sce@metadata$sample_name <- regmatches(basename(all_sce_files[x]), regexpr("[^\\.]+[^\\.]+", basename(all_sce_files[x])))
  print(my_sce@metadata$sample_name)
  my_sce
})
cat("\n\nPrint resources:\n")
cat("\nall_sce_objects:\n")
print(object_size(all_sce_objects))
cat("\nmem_used():\n")
print(mem_used())


all_sampleIDs <- sapply(all_sce_files, function(x) regmatches(basename(x), regexpr("[^\\.]+[^\\.]+", basename(x))))

# optionally, read in an additional meta data table
print(empty_string)
if (is.character(opt$add_meta)) {
  addMeta_table <- read.csv(opt$add_meta, sep = "\t", stringsAsFactors = FALSE)
  print("Reading in meta data table: check if input samples and meta data sampleIDs are identical")
  print(sort(addMeta_table$sampleID))
  print(sort(all_sampleIDs))
  stopifnot(sort(addMeta_table$sampleID) == sort(all_sampleIDs))
} else if (is.null(opt$add_meta)) {
  print("No input file with additional meta data given.")
  addMeta_table <- NULL
} else {
  print("Something went wrong reading in the meta data table.")
}

# optionally, use only cells of one (major) cell type
all_celltypes <- c("Melanoma.melanocytic", "Melanoma.mesenchymal",
                   "Plasmacytoid.dendritic.cell",
                   "uncertain", "unknown",
                   "T.cells", "B.cells",
                   "Macrophages", "Plasma.cells",
                   "Endothelial.cells", "CAFs", "Dendritic.cells")
if (!is.null(opt$subset_celltype)) {
  cat("\n\n\nOnly processing cells of type:", opt$subset_celltype, "\n\n")
  keep_celltypes <- all_celltypes[grepl(opt$subset_celltype, all_celltypes, ignore.case = T)]
  print(str(keep_celltypes))
  print(keep_celltypes)
  my_sample_name <- opt$outName %&% "." %&% opt$subset_celltype
  cat("\n\nNew sample output name:", my_sample_name, "\n\n")
} else {
  cat("\n\n\nAll cells are being processed\n\n")
  keep_celltypes <- all_celltypes
  print(str(keep_celltypes))
  my_sample_name <- opt$outName %&% ".all.cells"
  cat("\n\nNew sample output name:", my_sample_name, "\n\n")
}




# get list of all cell description columns (colData) in all samples to check if
# GEX cell description data is identical between samples
print("Checking colData")
# get character vector with all colData headers of all samples
all_colData_headers <- character()
for (sample in seq_along(all_sce_objects)) {
  all_colData_headers <- c(all_colData_headers, names(colData(all_sce_objects[[sample]])))
}

# get frequency of each colData header
table_headers <- table(all_colData_headers)
# identify columns that are not represented in all samples
header_not_all_samples <- table_headers != length(all_sce_objects)
headers_remove <- table_headers[header_not_all_samples]
cat("\n\n\nWARNING: removing colData columns that are not found in all samples:\n\n")
print(headers_remove)
names_headers_remove <- names(headers_remove)

# remove those columns from all samples
for (sample in seq_along(all_sce_objects)) {
  for (header in names_headers_remove) {
    print(header)
    print(str(header))
    colData(all_sce_objects[[sample]])[header] <- NULL
  }
}


# check if now all colData columns are in all samples
headers_after_filter <- character()
for (sample in seq_along(all_sce_objects)) {
  print(sample)
  headers_after_filter <- c(headers_after_filter, names(colData(all_sce_objects[[sample]])))
}
# get frequency of each colData header
table_headers_after <- table(headers_after_filter)
# identify columns that are not represented in all samples
header_not_all_after <- table_headers_after != length(all_sce_objects)
cat("\n\nCheck if the cell description columns in all samples are identical.\n")
stopifnot(sum(header_not_all_after) == 0)
unique_cell_desc <- unique(headers_after_filter)




###   combine all SCE objects   ###
combined <- scMerge::sce_cbind(all_sce_objects,
                      exprs = c("counts", "normcounts", "pearson_resid"),
                      method = "union",
                      cut_off_batch = 0.0,
                      cut_off_overall = 0.0,
                      colData_names = unique_cell_desc)



# find duplicated barcodes
barcodes_no_prefix <- gsub(pattern = "[^-]+-(.*)", "\\1", colnames(combined))
indices_duplicated <- which(duplicated(barcodes_no_prefix))
barcodes_duplicated <- barcodes_no_prefix[indices_duplicated]
indices_ALL_duplicated <- which(barcodes_no_prefix %in% barcodes_duplicated)
mask_ALL_duplicated <- barcodes_no_prefix %in% barcodes_duplicated
cat("\n\nDuplicated barcodes:\n")
print(table(mask_ALL_duplicated))
cat("\n")

# somehow R turns hyphens in column headers into dots (when using as.data.frame())
# Therefore all hyphens are replaced by underscores.
names_without_hyphen <- gsub("-", "_", names(colData(combined)))
colData_df <- as.data.frame(colData(combined))
names(colData_df) <- names_without_hyphen
# add column with duplicated barcodes
colData_df$duplicated_barcode <- mask_ALL_duplicated




genecounts_meso = rowSums(counts(combined))
cellcounts_meso = colSums(counts(combined))
checkcounts_meso = apply(counts(combined), 2, function(x) length(which(x>0)))

meso_sub_2 = combined[!is.na(genecounts_meso) & genecounts_meso > 40, cellcounts_meso > 0 & checkcounts_meso > 0]  # some genes removed

#Change the option for the max size allowed for globals exported to the future expression as the results of the vst function are slightly higher than the default
options(future.globals.maxSize = 891289600)

vst_out_meso = sctransform::vst(counts(meso_sub_2), cell_attr = colData(meso_sub_2), method="nb_fast", 
                             latent_var = c('log_umi'), latent_var_nonreg = c("g2m_score", "s_score"), 
                             return_gene_attr = T, return_cell_attr = T)

print("Start performing sctransform::smooth_via_pca: ")
y_smooth_meso = sctransform::smooth_via_pca(vst_out_meso$y, do_plot = TRUE)

print("Start performing sctransform::correct: ")
dat_cor_meso = sctransform::correct(vst_out_meso, data = y_smooth_meso,
                               do_round = TRUE, do_pos = FALSE)


# Plot mean variance plot
mvp_meso <- ggplot(vst_out_meso$gene_attr, aes(log10(gmean), log10(residual_variance))) +
  geom_point(alpha=0.3, shape=16) +
  geom_smooth()
ggsave(opt$outdir %&% opt$outName %&% ".mesothelial.mean_vs_variance_plot.png", mvp_meso, dpi = 300)
mvp2_meso <- ggplot(vst_out_meso$gene_attr, aes(log10(gmean), residual_variance)) +
  geom_point(alpha=0.3, shape=16) +
  geom_density_2d(size = 0.3) +
  geom_smooth()
ggsave(opt$outdir %&% opt$outName %&% ".mesothelial.mean_vs_variance_plot_2.png", mvp2_meso, dpi = 300)

## make SingleCellExperiment object and save to disk
vst_out_meso$gene_attr$gene_ids = rownames(vst_out_meso$gene_attr)
rowData(meso_sub_2)$gene_ids = rownames(meso_sub_2)

gene_desc = merge(vst_out_meso$gene_attr, rowData(meso_sub_2), sort=F, by = "gene_ids")
idx = match(rownames(dat_cor_meso), rownames(meso_sub_2))
idy = match(colnames(dat_cor_meso), colnames(meso_sub_2))
sce_meso = SingleCellExperiment(assays = list(counts=meso_sub_2[idx, idy],
                                         normcounts=dat_cor_meso,
                                         pearson_resid=vst_out_meso$y),
                           colData = colData(meso_sub_2),
                           rowData = gene_desc)

saveRDS(sce_meso, opt$outdir %&% opt$outName %&% ".mesothelial.subtype.RDS")



library(SingleCellExperiment)
library(ggplot2)
library(scMerge)
library(uwot)
library(patchwork)
library(optparse)
library(reshape2)
library(igraph)
library(ggrepel)
library(scran)
library(dynamicTreeCut)
library(RColorBrewer)
library(pheatmap)


subtype_sce <- sce_meso


#Load the original seurat oject
seurat_integrated=readRDS("Tcell_new_seurat_integrated_wADT_fromscratch.RDS")

#confirm that all barcodes from the seurat object are in subtype_sce
tmp=unlist(lapply(strsplit(seurat_integrated$barcodes, split="_"), function(x)x[1]))
which(!tmp%in%subtype_sce$barcodes)
#integer(0)


#Create a new subset of subtype_sce with only the cells of the seurat object before going on with the script
tmp2=which(colnames(subtype_sce) %in% tmp)
subtype_sce_bck=subtype_sce
subtype_sce=subtype_sce[,tmp2]

subtype_sce$barcodes_v2 = tmp2

#Create a new subset of subtype_sce with only the cells of the seurat object before going on with the script
tmp2=which((subtype_sce$barcodes_v2) %in% seurat_integrated$barcodes)
subtype_sce_bck2=subtype_sce
subtype_sce=subtype_sce[,tmp2]

subtype_sce$seurat_clusters=NA
tmp=data.frame(barcodes=as.character(seurat_integrated$barcodes), seurat_clusters=as.character(seurat_integrated$seurat_clusters))
missing_cluster_num=0
missing_cluster_list=NULL
for(i in 1:length(subtype_sce$barcodes_v2)){
	ttt = which(tmp$barcodes %in% subtype_sce$barcodes_v2[i])
	if(length(ttt)>1){stop()}
	if(length(ttt)==0){
		missing_cluster_num = missing_cluster_num +1
		missing_cluster_list= c(missing_cluster_list, subtype_sce$barcodes_v2[i])
		next()
	}
	subtype_sce$seurat_clusters[i] = tmp$seurat_clusters[ttt]
	if(i %% 100 == 0 ){ message(paste(i, "of", length(subtype_sce$barcodes_v2)))}
}
message(paste("found", missing_cluster_num, "barcodes without clusters. Expecting", length(which(! subtype_sce$barcodes_v2%in%seurat_integrated$barcodes))))

#Test if there are still clusters with NA
which(is.na(subtype_sce$barcodes_v2))

max_hvg_length <- 1000

####### first compute the highly variable genes again

meanvartrend = modelGeneVar(normcounts(subtype_sce))
hvg = getTopHVGs(meanvartrend, fdr.threshold = 0.05)
length(hvg)
head(hvg)

gene_desc_subtype = meanvartrend
gene_desc_subtype$symbol = rownames(meanvartrend)
gene_desc_subtype$hvg = (gene_desc_subtype$symbol %in% hvg)

#show hvgs
ggplot(as.data.frame(gene_desc_subtype), aes(x=mean, y=total)) + 
  geom_point(aes(color=hvg)) + geom_smooth() +
  geom_line(aes(x=mean, y=tech), color="darkgreen", size=2) 
filename <- paste0(opt$outdir, opt$outName, "_hvg_mean_var.png")
ggsave(filename = filename, width = 30, height = 20, dpi = 300, units = "cm")

# pick max number of hvg's
dtest = gene_desc_subtype[hvg, ]
dtest = dtest[order(dtest$bio, decreasing = T), ]    
dtest = dtest[seq_len(min(nrow(dtest), max_hvg_length)), ]
gene_desc_subtype[setdiff(rownames(gene_desc_subtype), rownames(dtest)), ]$hvg = FALSE
hvg = rownames(gene_desc_subtype)[which(gene_desc_subtype$hvg)]

normcounts_all.zero.removed <- normcounts(subtype_sce)
mask_all_zero <- apply(normcounts_all.zero.removed, 1, sum) > 0
normcounts_all.zero.removed <- normcounts_all.zero.removed[mask_all_zero, ]
stopifnot(length(normcounts_all.zero.removed[, 1]) == sum(apply(normcounts_all.zero.removed, 1, sum) > 0))

all_umap_clusters <- unique(colData(subtype_sce)$seurat_clusters)
print("seurat clusters:")
print(all_umap_clusters)

all_cluster_means <- as.data.frame(rownames(subtype_sce))

names(all_cluster_means) <- "gene"
all_cluster_means$gene <- as.character(all_cluster_means$gene)

# Use relative counts normalisation before calculating mean expression per cluster
# get mean expression for each umap cluster
for (cluster_ID in all_umap_clusters) {
  print(paste0("Calculating mean expression for cluster ", cluster_ID))
  col_header <- paste0("clust_", cluster_ID)
  print(col_header)
  subset_cluster <- subtype_sce[, subtype_sce$seurat_clusters == cluster_ID]
  print(paste0("Number of cells in cluster: ", dim(subset_cluster)[2]))
  stopifnot(all_cluster_means$gene == rownames(subset_cluster))
  
  all_cluster_means[col_header] <- apply(assay(subset_cluster, "normcounts"), 1, mean)
  all_cluster_means[col_header] <- round(all_cluster_means[col_header], digits = 4)
  all_cluster_means[col_header][all_cluster_means[col_header] < 0] <- 0
}
# write mean expression table
filename_mean_expr <- paste0(opt$outdir, opt$outName, ".mean_expression_per_subtype_seurat_cluster.tsv")
write.table(all_cluster_means, file = filename_mean_expr, quote = FALSE, sep = "\t", row.names = FALSE)

### HEATMAP CLUSTER MEAN EXPRESSION
### TOP 50 and TOP 100 HVG
dtest50 = dtest[seq_len(min(nrow(dtest), 50)), ]
subset_hvg50 = subset(all_cluster_means, gene %in% rownames(dtest50))

rownames(subset_hvg50) = subset_hvg50$gene

dropCol = c("gene")
subset_hvg50_df =  subset_hvg50[,!(names(subset_hvg50) %in% dropCol)]
subset_hvg50_transform = subset_hvg50_df + 1
subset_hvg50_log = log10(subset_hvg50_transform)

hm_hvg50 = pheatmap(subset_hvg50_df, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_top50_hvg.png")
ggsave(filename = filename, hm_hvg50, width = 20, height = 40, dpi = 300, units = "cm")

hm_hvg50_log = pheatmap(subset_hvg50_log, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_top50_hvg_log.png")
ggsave(filename = filename, hm_hvg50_log, width = 20, height = 40, dpi = 300, units = "cm")

dtest100 = dtest[seq_len(min(nrow(dtest), 100)), ]
subset_hvg100 = subset(all_cluster_means, gene %in% rownames(dtest100))

rownames(subset_hvg100) = subset_hvg100$gene

dropCol = c("gene")
subset_hvg100_df =  subset_hvg100[,!(names(subset_hvg100) %in% dropCol)]
subset_hvg100_transform = subset_hvg100_df + 1
subset_hvg100_log = log10(subset_hvg100_transform)

hm_hvg100 = pheatmap(subset_hvg100_df, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_top100_hvg.png")
ggsave(filename = filename, hm_hvg100, width = 20, height = 60, dpi = 300, units = "cm")

hm_hvg100_log = pheatmap(subset_hvg100_log, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_top100_hvg_log.png")
ggsave(filename = filename, hm_hvg100_log, width = 20, height = 60, dpi = 300, units = "cm")

#heatmap on the shortlist of genes provided
mygenes <- read.delim("genes.txt")

which(rownames(dtest)%in%mygenes[,1])
#[1]   4   9  11  13  17  18  20  24  32  42  51  53  56  59  61  69  71  81  83
#[20] 118 180

#only 21 genes of the list are present in the dtest object
# That means only 21 genes from the list are highly variable. I don't have to use dtest nor to do any testing, I just need
# to plot norm counts for this list

subset_custom = subset(all_cluster_means, gene %in% mygenes[,1])
rownames(subset_custom) <- subset_custom$gene
dropCol = c("gene")
subset_custom_df = subset_custom[,!(names(subset_custom) %in% dropCol)]
subset_custom_transform = subset_custom_df + 1
subset_custom_log = log10(subset_custom_transform)

hm_custom <- pheatmap(subset_custom_df, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_custom.png")
ggsave(filename = filename, hm_custom, width = 20, height = 60, dpi = 300, units = "cm")

hm_custom_log = pheatmap(subset_custom_log, scale="none", cluster_rows = T, clustering_method = "ward.D2",
         show_rownames = T, show_colnames = T, fontsize=8, treeheight_col = 50)
filename <- paste0(opt$outdir, opt$outName, "_heatmap_custom_log.png")
ggsave(filename = filename, hm_custom_log, width = 20, height = 60, dpi = 300, units = "cm")


