#_################################################
## File name: integrateSamples_analyseCohort.R
## Author: Anne Bertolini
## Date created: April 2020
## R Version: 3.5.1
##################################################

## GENERAL:
## This script is a template script for single cell integration analyses using the Seurat R package.
## Several changes will be required to adapt the output to the project at hand. Please find extensive documentation below.

### PARTICULAR SCRIPT
## This particular version of the script can
# - reduce the data to one cell tpye given as command line parameter


## INPUT:
## - script takes list of RDS files (that contain SCE objects containing results of an
##   scRNA-seq experiment) and integrates those samples into one integrated Seurat object
##   (using the R package Seurat).
## - script takes second list of RDS files that contain results of GSVA analyses of
##   the same samples. IMPORTANT: Both lists have to be in the same order.
## - script can optionally work with an additional table of meta data about the samples.
## --> with package optparse for parsing command line arguments no parameter is strictly needed. If there is no meta data table to add simply do not use --add_meta
## For more details regarding input see help phrases of command line arguments.

##   META DATA FILE:
## Example:
##   sampleID     | cancerStage | pctRibosomalReads | reference
##   ----------------------------------------------------------
##   sample_1.RDS | stage III   | 0.3               | reference
##   sample_2.RDS | stage IV    | 0.1               | other
## If a plot should be generated from a meta data column the respective code block can be added in the Sample Visualisation section of the script.
## The Seurat function DimPlot() is used for categorical data and the function FeaturePlot() for continuous data.
## Note that DimPlot() does not plot a legend title using default settings (see additional line in existing code).

## OUTPUT:
## - integrated Seurat object in an RDS file.
## - UMAP plots showing GSVA results
## - UMAP plots characterising the cohort of samples. The plots include all integrated cells in one UMAP
##   and visualise data like cell types or g2m_score, n_umi, fractionMT.
## - UMAP plots showing gene expression
## - mean expression table per seurat cluster

## REMARK sampleID:
## The default is that the whole file name is entered as the sampleID (automatically retrieved from the input files and entered by the user in the meta data table).
## This is to keep this script very general and functioning without initial changes. But, especially for the plot
## showing the UMAP with all cells coloured by sampleID it is not optimal. If a shorter sampleID is desired you find example code with
## regular expressions to get a substring of the file name and have it serve as the 'sampleID'. This regular expression has to be changed and activated in two places
## indicated with 1) and 2).


suppressPackageStartupMessages({
  library(Seurat)
  library(plyr)
  library(dplyr)
  library(SingleCellExperiment)
  library(optparse)
  library(reticulate)
  library(ggplot2)
  library(reshape2)
  library(cowplot)
  library(Matrix)
  library(pryr)
})

# give out session Info
cat("\n\n\nPrint sessionInfo:\n\n")
print(sessionInfo())
cat("\n\n\n\n")

# parse command line arguments
option_list <- list(
  make_option("--cohort_list", type = "character", help = "List with paths to the input RDS files that contain SCE objects with GEX results. INPORTANT: GEX and GSVA input files have to be in the same order."),
  make_option("--colour_config", type = "character", help = "Path to config file table that sorts colours to cell types"),
  make_option("--selectedGenes", type = "character", help = "Tab delimited text file with the genes of interest in the first column. No header. The genes will be plotted and given out in chunks, alphabetical order."),
  make_option("--add_meta", type = "character", help = "Tab delimited text file with additional meta data. The first column must have header 'sampleID'. For a default run the full file name of the respective SCE analysis object is required, refer to script documentation on different sampleID formatting. Column headers describe categories. One line per input sample is required. Meta data file itself is optional: without table simply do not use --add_meta"),
  make_option("--subset_celltype", type = "character", help = "Use a subset of the cells, this (major) cell type only"),
  make_option("--outdir", type = "character", help = "Full path to output directory."),
  make_option("--outName", type = "character", help = "Prefix name of output files.")
)
opt_parser <- OptionParser(option_list = option_list)
opt <- parse_args(opt_parser)


# convenience function for string concatenation
"%&%" <- function(a, b) paste(a, b, sep = "")
empty_string <- ""


###################################
###   Set plotting Parameters   ###
###################################

# If one of the values plot_aspect_ratio, chunksize, or number of columns is changed in the following section
# the other values should be checked as well.
# Also, check physical size of the generated output file in the respective plotting section of the script.

# IMPORTANT: when generating UMAP plots with the function RunUMAP() from Seurat the parameters "spread", "min.dist" and "n.neighbors"
# have a great impact on the resulting plot. They should be checked and optimised for each project.

# for all UMAP plots the same aspect ratio is used to have a comparable visualisation
plot_aspect_ratio <- 0.7

### The selected genes and gene sets are plotted in chunks so that the file size is not too large:

# chunksize_genes determines the number of genes shown in one expression plot
chunksize_genes <- 9
# columns_genes determines the number of columns the gene expression plots are arranged in.
## E.g. "columns_genes <- 1" arranges all plots in the output file below one another in a single column.
columns_genes <- 3

# chunk size of gene sets in GSVA plots
chunksize_genesets <- 12
# number of columns in GSVA plots
columns_genesets <- 4

# override the theme change of cowplot
theme_set(theme_grey())
print(Sys.time())


# parameter when using only subset of cell types
# each sample must contain a minimum number of cells of this cell type to be included
# in the cell type specific analysis
min_cells_per_celltype <- 30

# The number of neighbours considered when filtering the anchors for the integration cannot be
# higher than the number of total cells in this sample
# The default is 200
number_neighbours_filtering <- 20

########################
###   Read in data   ###
########################

# read in data from all samples of the cohort
cohort_files <- read.csv(opt$cohort_list, header = FALSE, stringsAsFactors = FALSE)
all_sce_files <- cohort_files$V1
# make a list of all SCE objects
all_sce_objects <- lapply(seq_along(all_sce_files), function(x) {
  print(paste0("Read in sample ", x))
  print(all_sce_files[x])
  my_sce <- readRDS(all_sce_files[x])

  # 1) example regex expression to shorten the sample name
  # example sample name is the basename of the input file up to (but excluding) the second "."
  # e.g. IDAMEME-T_scR_Ar1v7.6.genes_cells_filtered.corrected.atypical_removed.RDS --> IDAMEME-T_scR_Ar1v7.6
  #my_sce@metadata$sample_name <- regmatches(basename(all_sce_files[x]), regexpr("[^\\.]+\\.[^\\.]+", basename(all_sce_files[x])))

  # default sample name is the basename of the input file
  my_sce@metadata$sample_name <- regmatches(basename(all_sce_files[x]), regexpr("[^\\.]+[^\\.]+", basename(all_sce_files[x])))
  print(my_sce@metadata$sample_name)
  my_sce
})
cat("\n\nPrint resources:\n")
cat("\nall_sce_objects:\n")
print(object_size(all_sce_objects))
cat("\nmem_used():\n")
print(mem_used())

# have number of samples
number_samples <- length(all_sce_files)
all_sampleIDs <- sapply(all_sce_files, function(x) regmatches(basename(x), regexpr("[^\\.]+[^\\.]+", basename(x))))

# optionally, read in an additional meta data table
print(empty_string)
if (is.character(opt$add_meta)) {
  addMeta_table <- read.csv(opt$add_meta, sep = "\t", stringsAsFactors = FALSE)
  print("Reading in meta data table: check if input samples and meta data sampleIDs are identical")
  print(sort(addMeta_table$sampleID))
  print(sort(all_sampleIDs))
  stopifnot(sort(addMeta_table$sampleID) == sort(all_sampleIDs))
} else if (is.null(opt$add_meta)) {
  print("No input file with additional meta data given.")
  addMeta_table <- NULL
} else {
  print("Something went wrong reading in the meta data table.")
}


# optionally, use only cells of one (major) cell type
all_celltypes <- c("Melanoma.melanocytic", "Melanoma.mesenchymal",
                   "Plasmacytoid.dendritic.cell",
                   "uncertain", "unknown",
                   "T.cells", "B.cells",
                   "Macrophages", "Plasma.cells",
                   "Endothelial.cells", "CAFs", "Dendritic.cells")
if (!is.null(opt$subset_celltype)) {
  cat("\n\n\nOnly processing cells of type:", opt$subset_celltype, "\n\n")
  keep_celltypes <- all_celltypes[grepl(opt$subset_celltype, all_celltypes, ignore.case = T)]
  print(str(keep_celltypes))
  print(keep_celltypes)
  my_sample_name <- opt$outName %&% "." %&% opt$subset_celltype
  cat("\n\nNew sample output name:", my_sample_name, "\n\n")
} else {
  cat("\n\n\nAll cells are being processed\n\n")
  keep_celltypes <- all_celltypes
  print(str(keep_celltypes))
  my_sample_name <- opt$outName %&% ".all.cells"
  cat("\n\nNew sample output name:", my_sample_name, "\n\n")
}


###################################
###   Generate Seurat objects   ###
###################################

# Using Seurat SCTransform for sample integration
# Following the Vignette https://satijalab.org/seurat/v3.1/integration.html#SCTransform

# generate a list of Seurat objects from the list of SCE objects
# Fill the Seurat objects with the raw counts, gsva results, and metadata
print(empty_string)
all_samples <- lapply(seq_along(all_sce_objects), function(x) {
  print(paste0("Build seurat object of ", x, ". SCE object:"))
  # make sure that no cell and no gene (feature) is lost to default thresholds
  seurat_obj <- CreateSeuratObject(counts = assay(x = all_sce_objects[[x]], "counts"), min.cells = 0, min.features = 0)

  # create adt assay
  # get matrix containing only ADT counts
  # somehow R turns hyphens in column headers into dots (when using as.data.frame())
  # Therefore all hyphens are replaced by underscores.
  names_without_hyphen <- gsub("-", "_", names(colData(all_sce_objects[[x]])))
  colData_df <- as.data.frame(colData(all_sce_objects[[x]]))
  names(colData_df) <- names_without_hyphen
  mask_ADT_cols_temp <- names(colData_df)
  # Rename the col ADT_barcodes, to avoid including it latere on.
  mask_ADT_cols_temp <- replace(mask_ADT_cols_temp, mask_ADT_cols_temp=="ADT_barcodes", "Barcodes_ADT")
  print(mask_ADT_cols_temp)
  mask_ADT_cols <- grepl("ADT_",  mask_ADT_cols_temp)
  print(mask_ADT_cols)
  print(names(colData_df)[mask_ADT_cols])
  mask_ADT_cols_including_HT <- names(colData_df)[mask_ADT_cols]
  
  mask_ADT_cols_noHT <- !grepl("Hashtag",  mask_ADT_cols_including_HT)
  print("mask_ADT_cols_noHT")
  print(mask_ADT_cols_noHT)
  print(mask_ADT_cols_including_HT[mask_ADT_cols_noHT])
  adt_counts <- subset(colData_df, select = mask_ADT_cols_including_HT[mask_ADT_cols_noHT])
  adt_features <- colnames(adt_counts)[-1]
  seurat_obj[["adt"]] <- CreateAssayObject(counts =t(as.matrix(adt_counts)))
  # include all the metadata from the SCE objects

  seurat_obj[["phenograph_clusters"]] <- colData(all_sce_objects[[x]])$phenograph_clusters
  seurat_obj[["celltype_major"]] <- colData(all_sce_objects[[x]])$celltype_major
  seurat_obj[["celltype_final"]] <- colData(all_sce_objects[[x]])$celltype_final
  seurat_obj[["fractionMT"]] <- colData(all_sce_objects[[x]])$fractionMT
  seurat_obj[["n_umi"]] <- colData(all_sce_objects[[x]])$n_umi
  seurat_obj[["n_gene"]] <- colData(all_sce_objects[[x]])$n_gene
  seurat_obj[["log_umi"]] <- colData(all_sce_objects[[x]])$log_umi
  seurat_obj[["g2m_score"]] <- colData(all_sce_objects[[x]])$g2m_score
  seurat_obj[["s_score"]] <- colData(all_sce_objects[[x]])$s_score
  seurat_obj[["cycle_phase"]] <- colData(all_sce_objects[[x]])$cycle_phase
  seurat_obj[["celltype_major_full_ct_name"]] <- colData(all_sce_objects[[x]])$celltype_major_full_ct_name
  seurat_obj[["celltype_final_full_ct_name"]] <- colData(all_sce_objects[[x]])$celltype_final_full_ct_name
  seurat_obj[["sample_name"]] <- all_sce_objects[[x]]@metadata$sample_name
  seurat_obj[["phenograph_cluster"]] <- colData(all_sce_objects[[x]])$phenograph_clusters
  # add additional meta data from input table
  if (is.null(addMeta_table)) {
    print("No additional meta data is read into the seurat objects.")
  } else if (is.data.frame(addMeta_table)) {
    cat("\nall_sce_objects[[x]]@metadata$sample_name:\n")
    cat("\n\n", all_sce_objects[[x]]@metadata$sample_name, "\n")
    subset_meta <- subset(addMeta_table, sampleID == all_sce_objects[[x]]@metadata$sample_name)
    cat("\n\ndim(subset_meta):\n")
    print(dim(subset_meta))
    print("Check if subset of the meta data table only contains one sample.")
    stopifnot(dim(subset_meta)[1] == 1)
    # for each column of meta data add a column to the seurat object's meta data
    for (column in names(subset_meta)) {
      seurat_obj[[column]] <- subset_meta[[column]]
    }
  } else {
    print("Something went wrong with the additional meta data table.")
  }
  # keep only cells of a specific cell type
  print(table(seurat_obj[["celltype_major"]]))
  subset_celltypes <- grepl(opt$subset_celltype, names(table(seurat_obj[["celltype_major"]])), ignore.case = T)
  number_cells <- sum(table(seurat_obj[["celltype_major"]])[subset_celltypes])
  # minimum number of cells of the respective cell type
  if (number_cells > min_cells_per_celltype) {
    seurat_obj <- subset(x = seurat_obj, subset = celltype_major %in% keep_celltypes)
    print(seurat_obj)
    print(table(seurat_obj[["celltype_major"]]))
    return(seurat_obj)
  } else {
    print("Warning: No or too few cells available in this sample!")
  }
})
cat("\n\nPrint resources:\n")
cat("\nall_samples::\n")
print(object_size(all_samples))
cat("\nmem_used():\n")
print(mem_used())
cat("\n\nRemove SCE objects:\n")
rm(all_sce_objects)
cat("\nmem_used():\n")
print(mem_used())

cat("\n\n\n#####     All samples included in integration:     #####\n\n")
print(all_samples)
print(unlist(lapply(all_samples, typeof)))
# remove list elements that are not Seurat objects
character_entries <- unlist(lapply(all_samples, typeof) == "character")
all_samples <- all_samples[!character_entries]
cat("\n\n\n#####     All samples included in integration after filtering:     #####\n\n")
print(all_samples)
print(unlist(lapply(all_samples, typeof)))

##################################
###   Integration of samples   ###
##################################

# Perform SCTransform on each of the new Seurat objects, separately:
cat("\n\nPerform SCTransform.\n")
print(Sys.time())
for (i in seq_len(length(all_samples))) {
  # default for latent_var is c("log_umi")
  all_samples[[i]] <- SCTransform(all_samples[[i]], verbose = FALSE, latent_var = c("log_umi"), vars.to.regress = c("g2m_score", "s_score"))
}


# Perform ADT Normalization on each of the ADT assay, seperately:
for (i in seq_len(length(all_samples))) {
  print("normalize adt")
  print(all_samples[[i]])
  all_samples[[i]] <- NormalizeData(all_samples[[i]], assay = "adt", normalization.method = "CLR", margin = 2)
  print("scale")
  all_samples[[i]] <- ScaleData(all_samples[[i]], assay = "adt")
  print("VariableFeatures")
  all_samples[[i]] <- FindVariableFeatures(all_samples[[i]], assay = "adt", selection.method = "vst") 
}


# Select integration features and prepare integration
print(empty_string)
print("Prepare integration GEX.")
print(Sys.time())





###################################################################################################################
###     Data integration with reference     ###
### data can be integrated using a reference data set. This reduces substantially the resources that are needed
### and can be the only option if large data sets are integrated, or if among the samples there are two or more
### samples that do not have any overlap in cell types.
### In this case the additional input meta data table must contain a column "reference" where the reference samples
### have the string "reference" and the others e.g. "other".
### Comment in the following lines and use the parameter "reference" in the function FindIntegrationAnchors
###################################################################################################################

# The number of features might have to be adapted per project, e.g. reduced if many samples are integrated or if no reference is used
# only those integration features are integrated
integration_features <- SelectIntegrationFeatures(object.list = all_samples, nfeatures = 500)
all_samples <- PrepSCTIntegration(object.list = all_samples, anchor.features = integration_features)
# Find integration anchors
cat("\n\nFind integration anchors.\n")
print(Sys.time())

integration_anchors <- FindIntegrationAnchors(object.list = all_samples, normalization.method = "SCT",
                                  anchor.features = integration_features,
                                  k.filter = number_neighbours_filtering)
# Integrate samples
cat("\n\nIntegrate samples.\n")
print(Sys.time())
seurat_integrated <- IntegrateData(anchorset = integration_anchors, normalization.method = "SCT", k.weight = min_cells_per_celltype -1, new.assay.name = "integratedGEX")


# integrate ADT
for (i in seq_len(length(all_samples))) {
DefaultAssay(object = all_samples[[i]]) <- "adt"
}

print("integrate ADT")
print("Find integration anchors ADT")
integration_anchors <- FindIntegrationAnchors(object.list = all_samples)#,
print("Integrate samples ADT")
#### WARNING: CHANGED THE k.weight PARAMETER TO ALLOW THE FUNCTION NOT TO CRASH
#I set it to the highest possible amount that worked
#According to: https://github.com/satijalab/seurat/issues/3930
#It should not be a problem and the error is mostly related to how noisy the sample is
seurat_integrated_temp <- IntegrateData(anchorset = integration_anchors, new.assay.name = "integratedADT", k.weight=41)
print(str(seurat_integrated_temp))

integratedADT_data <- GetAssayData(seurat_integrated_temp, slot = "data", assay = "integratedADT")
dim(integratedADT_data)

# The following couple of lines have become necessary due to a Seurat Bug. 
# Could probably be written clearer once bug is resolved.
mat <- as.matrix(integratedADT_data)
mat <- mat[,colnames(seurat_integrated)]
features <- rownames(mat)


seurat_integrated_old <- readRDS("T_cells..T.cells.integrated_seurat_object.RDS")

seurat_integrated_old[["integratedADT"]] <- CreateAssayObject(data= mat)
CreateAssayObject(data = mat)
print(str(seurat_integrated_old))
### Run normalisation (scale), PCA and UMAP, find neighbours and clusters on ADT.
print(empty_string)
print("Run normalisation (scale), PCA and UMAP, find neighbours and clusters on ADT.")
print(Sys.time())
DefaultAssay(object = seurat_integrated_old) <- "integratedADT"
seurat_integrated_old <- ScaleData(seurat_integrated_old, verbose = FALSE)
seurat_integrated_old <- RunPCA(object = seurat_integrated_old, verbose = TRUE, reduction.name = 'apca', features = features)

seurat_integrated_old <- RunUMAP(object = seurat_integrated_old, reduction = "apca", dims = 1:40)
seurat_integrated_old <- FindNeighbors(object = seurat_integrated_old, reduction = "apca")
seurat_integrated_old <- FindClusters(seurat_integrated_old, dims.use = 1:40)
seurat_integrated_old[["barcodes"]] <- rownames(seurat_integrated_old[[]])

print(str(seurat_integrated_old))


mygenes <- read.delim("genes.txt")
mygenes <- mygenes[,1]

DoHeatmap(seurat_integrated_old, features=mygenes, assay="integrated")
ggsave(filename = paste0(opt$outdir, my_sample_name, ".heatmap_all.png"),
       width = 34, height = 20, dpi = 300, units = "cm")

###Now find the top 50 most variable genes

# see in the instructions if the runpca, runumap and all other next functions
# that do not explicitly call for adt, make use of adt data or not
# remember that just below we set the default assay back to the GEX,
#so you either need for the assay option to be set to integratedADT  or for the function
# to read the ADT data for any reason, hopefully specified in the instructions

cat("\n\nPrint resources:\n")
cat("\nseurat_integrated:\n")
print(object_size(seurat_integrated))
cat("\nmem_used():\n")
print(mem_used())
cat("\n\nRemove individual seurat objects:\n")
cat("\nmem_used():\n")
print(mem_used())

# run normalisation (scale), PCA and UMAP, find neighbours and clusters
print("\n\nRun normalisation (scale), PCA and UMAP, find neighbours and clusters.\n")
print(Sys.time())
DefaultAssay(object = seurat_integrated) <- "integratedGEX"

seurat_integrated <- ScaleData(seurat_integrated, verbose = FALSE)

seurat_integrated <- RunPCA(object = seurat_integrated, npcs = 20, verbose = TRUE)
seurat_integrated <- RunUMAP(object = seurat_integrated, reduction = "pca", dims = 1:20,
                            spread = 2, min.dist = 0.01, n.neighbors = 100)
seurat_integrated <- FindNeighbors(object = seurat_integrated)
seurat_integrated <- FindClusters(seurat_integrated)
seurat_integrated[["barcodes"]] <- rownames(seurat_integrated[[]])

# Find multimodal neighbors
seurat_integrated <- FindMultiModalNeighbors(
  seurat_integrated, reduction.list = list("pca", "apca"), 
  dims.list = list(1:50, 1:50), modality.weight.name = "RNA.weight"
)

seurat_integrated <- RunUMAP(seurat_integrated, nn.name = "weighted.nn", reduction.name = "wnn.umap", reduction.key = "wnnUMAP_")
seurat_integrated <- FindClusters(seurat_integrated, graph.name = "wsnn", algorithm = 3, resolution = 2, verbose = FALSE)

seurat_integrated <- RunUMAP(seurat_integrated, reduction = 'pca', dims = 1:50, assay = 'RNA', 
              reduction.name = 'rna.umap', reduction.key = 'rnaUMAP_')
seurat_integrated <- RunUMAP(seurat_integrated, reduction = 'apca', dims = 1:50, assay = 'ADT', 
              reduction.name = 'adt.umap', reduction.key = 'adtUMAP_')

# Write integrated Seurat object into RDS file
filename_out <- opt$outdir %&% my_sample_name %&% ".integrated_seurat_object.RDS"
print(paste0("File name of output RDS file with integrated Seurat object: ", filename_out))
saveRDS(seurat_integrated, filename_out)

# Write text file with seurat cluster ID and basic cell information
cell_attr_cluster <- subset(seurat_integrated[[]], select = c("sampleID", "barcodes", "seurat_clusters"))
filename_out <- opt$outdir %&% my_sample_name %&% ".seurat_cluster_info.txt"
cat("\n\nWrite seurat cluster info table:\n")
write.table(cell_attr_cluster, file = filename_out,
            quote = F, sep = "\t", row.names = F)

###############################
###   Sample Visualisation  ###
###############################

# read in colour_config and sort colours to cell types
config <- read.csv(opt$colour_config, sep = "\t", stringsAsFactors = FALSE)
config$cell_type <- gsub(pattern = "([^_]+)_.*", "\\1", config$cell_type)
print("Check if the number of cell types and the number of colours for the cell types match.")
stopifnot(length(config$colour) == length(config$cell_type))
stopifnot(config$colour == unique(config$colour))
# have character vector of colour names from the colour config
ct.color <- c(config$colour, "grey50", "black")
# give names to colours from other column of colour config
names(ct.color) <- c(config$cell_type, "uncertain", "unknown")
# make sure 'celltype_major' and 'celltype_final' metadata columns contain factors
seurat_integrated[["celltype_major"]] <- as.factor(seurat_integrated[[]]$celltype_major)
seurat_integrated[["celltype_final"]] <- as.factor(seurat_integrated[[]]$celltype_final)
# get integer vector containing the index of the matching colour in ct.color for each
# level of seurat_integrated[[]]$celltype_major
id.first.ct <- match(levels(seurat_integrated[[]]$celltype_major), names(ct.color))
id.first.ct
id.final.ct <- match(levels(seurat_integrated[[]]$celltype_final), names(ct.color))
id.final.ct

colstypes <- c("red2", "green4", "blue2", "cyan2", "yellow1", "purple", "brown",
               "chocolate1", "chartreuse2", "darkgoldenrod3", "steelblue1", "slateblue3", "olivedrab4", "gold2",
               "violetred3", "darkcyan", "orchid3", "darksalmon", "darkslategrey", "khaki", "indianred2", "magenta", "slategray2",
               "olivedrab1", "mediumaquamarine", "hotpink", "yellow3",
               "bisque4", "darkseagreen1", "dodgerblue3",
               "deeppink4", "sienna4", "mediumorchid4")


###   Plot general cohort composition   ###
print(empty_string)
print("Plot general cohort composition.")
print(Sys.time())

p1 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "sample_name") +
  theme(legend.position = "bottom", aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(ncol = 1, override.aes = list(size = 4), title = "sample_name"))
p1
ggsave(filename = paste0(opt$outdir, my_sample_name, ".sample_integration_all_samples.png"),
       width = 28, height = (22 + number_samples*0.3), dpi = 300, units = "cm")
print("Written plot 1")
###
p2 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "celltype_final", cols = ct.color[id.final.ct]) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "celltype_final", override.aes = list(size = 3)))
p2
ggsave(filename = paste0(opt$outdir, my_sample_name, ".sample_integration_celltypes.png"),
       width = 34, height = 20, dpi = 300, units = "cm")
print("Written plot 2")
###
p3 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "celltype_major", cols = ct.color[id.first.ct]) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "celltype_major", override.aes = list(size = 3)))
p3
ggsave(filename = paste0(opt$outdir, my_sample_name, ".sample_integration_major_celltypes.png"),
       width = 34, height = 20, dpi = 300, units = "cm")
print("Written plot 3")
###
p4 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "seurat_clusters", cols = colstypes) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "seurat_clusters", override.aes = list(size = 3)))
p4
ggsave(filename = paste0(opt$outdir, my_sample_name, ".sample_integration_seurat_umap_clusters.png"),
       width = 30, height = 20, dpi = 300, units = "cm")
print("Written plot 4")


###   Characterise cohort   ###
print(empty_string)
print("Characterise cohort plots.")
# Plot cycle phase
p22 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "cycle_phase", cols = colstypes) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "cycle_phase", override.aes = list(size = 3)))
p22
ggsave(filename = paste0(opt$outdir, my_sample_name, "_cellcycle_phase.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot fraction of MT reads
p23 <- FeaturePlot(seurat_integrated, features = c("fractionMT")) +
  theme(aspect.ratio = plot_aspect_ratio)
p23
ggsave(filename = paste0(opt$outdir, my_sample_name, "_fraction_MT.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot n_umi
p24 <- FeaturePlot(seurat_integrated, features = c("n_umi")) +
  theme(aspect.ratio = plot_aspect_ratio)
p24
ggsave(filename = paste0(opt$outdir, my_sample_name, "_n_umi.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot n_gene
p25 <- FeaturePlot(seurat_integrated, features = c("n_gene")) +
  theme(aspect.ratio = plot_aspect_ratio)
p25
ggsave(filename = paste0(opt$outdir, my_sample_name, "_n_gene.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot log_umi
p26 <- FeaturePlot(seurat_integrated, features = c("log_umi")) +
  theme(aspect.ratio = plot_aspect_ratio)
p26
ggsave(filename = paste0(opt$outdir, my_sample_name, "_log_umi.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot g2m_score
p27 <- FeaturePlot(seurat_integrated, features = c("g2m_score")) +
  theme(aspect.ratio = plot_aspect_ratio)
p27
ggsave(filename = paste0(opt$outdir, my_sample_name, "_g2m_score.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot s_score
p28 <- FeaturePlot(seurat_integrated, features = c("s_score")) +
  theme(aspect.ratio = plot_aspect_ratio)
p28
ggsave(filename = paste0(opt$outdir, my_sample_name, "_s_score.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot driver_gene
p30 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "driver_gene", cols = colstypes) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "driver_gene", override.aes = list(size = 3)))
p30
ggsave(filename = paste0(opt$outdir, my_sample_name, "_driver_gene.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot TMB_grouped
p31 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "TMB_grouped", cols = colstypes) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "TMB_grouped", override.aes = list(size = 3)))
p31
ggsave(filename = paste0(opt$outdir, my_sample_name, "_TMB_grouped.png"), width = 30, height = 20, dpi = 300, units = "cm")

# Plot biopsy_site_grouped
p32 <- DimPlot(seurat_integrated, reduction = "umap", group.by = "biopsy_site_grouped", cols = colstypes) +
  theme(aspect.ratio = plot_aspect_ratio) +
  guides(color = guide_legend(title = "biopsy_site_grouped", override.aes = list(size = 3)))
p32
ggsave(filename = paste0(opt$outdir, my_sample_name, "_biopsy_site_grouped.png"), width = 30, height = 20, dpi = 300, units = "cm")

###   Plot gene expression   ###
print(empty_string)
print("Gene expression plots.")
print(Sys.time())
# read in genes of interest
selectedGenes_file <- read.csv(opt$selectedGenes, header = FALSE,
                               stringsAsFactors = FALSE)
head(selectedGenes_file)
selectedGenes <- sort(unique(selectedGenes_file[[1]]))
chunks_selectedGenes <- split(selectedGenes, ceiling(seq_along(selectedGenes) / chunksize_genes))

# plot the expression values of sctransform normalised counts
# LogNormalize (for plotting): Feature counts for each cell are divided by the total counts for that cell
# and multiplied by the scale.factor. This is then natural-log transformed using log1p
DefaultAssay(seurat_integrated) <- "SCT"
seurat_integrated <- NormalizeData(seurat_integrated, normalization.method = "LogNormalize")

for (chunk in seq_along(chunks_selectedGenes)) {
  print(paste0("Chunk: ", chunk))
  print("Plotting chunk of genes of interest:")
  print(chunks_selectedGenes[[chunk]])
  print(str(chunks_selectedGenes[[chunk]]))
  genes_chunk <- selectedGenes[selectedGenes %in% chunks_selectedGenes[[chunk]]]
  p_selected_genes <- FeaturePlot(seurat_integrated, features = genes_chunk, ncol = columns_genes, combine = FALSE)
  # change aspect ratio in all ggplot list elements (plots)
  p_selected_genes <- lapply(p_selected_genes, function(x) x + theme(aspect.ratio = plot_aspect_ratio) + labs(color = "log"))
  plots_together <- plot_grid(plotlist = p_selected_genes, ncol = columns_genes)
  plots_together
  filename_geneEx <- paste0(opt$outdir, my_sample_name, ".integrated_gene_expression__", chunk, ".png")
  ggsave(filename = filename_geneEx,
         width = 45, height = 10 * ceiling(length(chunks_selectedGenes[[chunk]]) / columns_genes),
         dpi = 300, units = "cm")
}


# get mean expression per integrated cluster
###   TODO: Check if additional normalisation is necessary here!
all_seurat_clusters <- levels(seurat_integrated[[]]$seurat_clusters)
all_cluster_means <- as.data.frame(rownames(seurat_integrated))
names(all_cluster_means) <- "gene"
all_cluster_means$gene <- as.character(all_cluster_means$gene)
# Use relative counts normalisation before calculating mean expression per cluster
# RC: Feature counts for each cell are divided by the total counts for that cell
# and multiplied by the scale.factor. No log-transformation is applied.
DefaultAssay(seurat_integrated) <- "SCT"
seurat_integrated <- NormalizeData(seurat_integrated, normalization.method = "RC")
# Plot sum of all counts per cell and plot histogram
histo_data <- as.data.frame(colSums(GetAssayData(seurat_integrated, assay = "SCT", slot = "data")))
histo <- ggplot(histo_data, aes(x = histo_data[[1]])) +
  geom_histogram()
histo
filename_histo <- paste0(opt$outdir, my_sample_name, "_histogram_countsSums_per_cell.png")
ggsave(filename = filename_histo,
       width = 30, height = 20, dpi = 300, units = "cm")


# get mean expression for each seurat cluster
for (seurat_cluster_ID in all_seurat_clusters) {
  print(paste0("Calculating mean expression for cluster ", seurat_cluster_ID))
  col_header <- paste0("clust_", seurat_cluster_ID)
  print(col_header)
  subset_cluster <- subset(x = seurat_integrated, subset = seurat_clusters == seurat_cluster_ID)
  print(paste0("Number of cells in cluster: ", dim(subset_cluster)[2]))
  stopifnot(all_cluster_means$gene == rownames(subset_cluster))
  all_cluster_means[col_header] <- apply(X = GetAssayData(subset_cluster, assay = "SCT", slot = "data"), 1, mean)
  all_cluster_means[col_header] <- round(all_cluster_means[col_header], digits = 4)
}
# write mean expression table
filename_mean_expr <- paste0(opt$outdir, my_sample_name, ".mean_expression_per_seurat_cluster.tsv")
print(filename_mean_expr)
write.table(all_cluster_means, file = filename_mean_expr, quote = FALSE, sep = "\t", row.names = FALSE)
print(Sys.time())
